//PCX.CPP COPYRIGHT 1995 KEVIN HOFFMAN    
//ALL RIGHTS RESERVED

#include <io.h>
#include <fstream.h>
//Now include our header file for the pcx class
#include "pcx.hpp"

//This file is for reading/loading PCX files easily
//Note...Won't read an image > 64K because of those real
//mode segments :-(
    
//Default Constructor...
PCXFile::PCXFile(void)
{
    error=PCXFILE_ERROR_NONE;
    pcxload=0;
    pcxpicture=NULL;
}

//Default DeConstructor
PCXFile::~PCXFile(void)
{
    Close();
}

//Constructor To Load a PCX
PCXFile::PCXFile(char *filename)
{
    error=PCXFILE_ERROR_NONE;
    pcxload=0;
    pcxpicture=NULL;
    Load(filename);
}

//Loads a PCX File
short PCXFile::Load(char *filename)
{
    unsigned short width,height;
    unsigned long totalb=0;
    unsigned long index,totalrun;
    unsigned short data;
    fstream LoadFile;
    if (pcxload) Close();
    pcxload=0;
    //Try to open file
    LoadFile.open(filename,ios::in|ios::nocreate|ios::binary);
    if (!LoadFile.good())
    {
        error=PCXFILE_ERROR_LOADING;
        return error;
    }
    //Now Skip To The Width & Height stuff in header
    LoadFile.seekg(8,ios::beg);
    //Read in width & height
    LoadFile.read((char *)&width,2);
    LoadFile.read((char *)&height,2);
    width++;
    height++;
    PCXwidth=width;
    PCXheight=height;
    //Get total byte needed to store image
    totalb=(unsigned long)width*(unsigned long)height;
    //Get memory for image
    pcxpicture=(char *)new unsigned char[totalb+1];
    //Check for no memory
    if (pcxpicture==NULL)
    {
        error=PCXFILE_ERROR_NOMEM;
        return error;
    }
    //Ok...Now skip to 128 bytes after(the header) and read/decompress RLE
    //data into our buffer(now allocated)
    LoadFile.seekg(128,ios::beg);
    index=0;
    while (index<totalb)
    {
        //Get the byte
        data=(unsigned short)LoadFile.get();
        //If below 192 then is just a byte(not compressed)
        if ((data>=192)&&(data<=255))
        {
            totalrun=data-192;
            data=(unsigned short)LoadFile.get();
            while (totalrun-- >0)
            {
                pcxpicture[index++]=(unsigned char)data;
            }
        }
        else
        {
            pcxpicture[index++]=(unsigned char)data;
        }
    }
    //Now read in the palette
    LoadFile.seekg(-768L,ios::end);
    for (index=0; index<768; index++)
    {
        //Remember we have to shift to 6 bits because
        //VGA registers are only 6 bits :-)
        PCXpalette[(unsigned short)index]=(unsigned char)(LoadFile.get()>>2);
    }
    //Close the file
    LoadFile.close();
    //Show that we loaded the pcx file
    pcxload=1;
    return PCXFILE_ERROR_NONE;
}
    
//Closes a PCX File(deinits memory...Called by contructor
void PCXFile::Close(void)
{
    if (pcxload)
    {
        delete [] pcxpicture;
        pcxload=0;
        PCXwidth=0;
        PCXheight=0;
    }
}

//Returns the pointer to the picture
//Is null if no picture is loaded
char *PCXFile::PicturePtr(void)
{
    if (pcxload) return (char *)pcxpicture;
    else return NULL;
}

//Returns the pointer to the palette
//Is null if no picture is loaded
char *PCXFile::PalettePtr(void)
{
    return (char *)&PCXpalette;
}

//Returns then resets the error
short PCXFile::Error(void)
{
    short errorv=error;
    error=0;
    return error;
}
